@@ -35,6 +35,44 @@ class BasicBlock(_bytecode._InstrList):
 
             yield instr
 
+    def __getitem__(self, index):
+        value = super().__getitem__(index)
+        if isinstance(index, slice):
+            value = type(self)(value)
+            value.next_block = self.next_block
+
+        return value
+
+    def copy(self):
+        new = type(self)(super().copy())
+        new.next_block = self.next_block
+        return new
+
+    def legalize(self, first_lineno):
+        """Check that all the element of the list are valid and remove SetLineno.
+
+        """
+        lineno_pos = []
+        set_lineno = None
+        current_lineno = first_lineno
+
+        for pos, instr in enumerate(self):
+            if isinstance(instr, SetLineno):
+                set_lineno = current_lineno = instr.lineno
+                lineno_pos.append(pos)
+                continue
+            if set_lineno is not None:
+                instr.lineno = set_lineno
+            elif instr.lineno is None:
+                instr.lineno = current_lineno
+            else:
+                current_lineno = instr.lineno
+
+        for i in reversed(lineno_pos):
+            del self[i]
+
+        return current_lineno
+
     def get_jump(self):
         if not self:
             return None
@@ -53,40 +91,34 @@ def _compute_stack_size(block, size, maxsize):
     if block.seen or block.startsize >= size:
         return maxsize
 
+    def update_size(delta, size, maxsize):
+        size += delta
+        if size < 0:
+            msg = 'Failed to compute stacksize, got negative size'
+            raise RuntimeError(msg)
+        maxsize = max(maxsize, size)
+        return size, maxsize
+
     block.seen = True
     block.startsize = size
 
     for instr in block:
-
         if isinstance(instr, SetLineno):
             continue
 
-        size += instr.stack_effect
-        maxsize = max(maxsize, size)
-
-        if size < 0:
-            msg = 'Failed to compute stacksize, got negative size'
-            raise RuntimeError(msg)
-
         if instr.has_jump():
-            target_size = size
-
-            if instr.name == 'FOR_ITER':
-                target_size = size - 2
-
-            elif instr.name in {'SETUP_FINALLY', 'SETUP_EXCEPT'}:
-                target_size = size + 3
-                maxsize = max(target_size, maxsize)
-
-            elif instr.name.startswith('JUMP_IF'):
-                size -= 1
-
-            maxsize = _compute_stack_size(instr.arg, target_size, maxsize)
+            # first compute the taken-jump path
+            taken_size, maxsize = update_size(instr.stack_effect(jump=True),
+                                              size, maxsize)
+            maxsize = _compute_stack_size(instr.arg, taken_size, maxsize)
 
             if instr.is_uncond_jump():
                 block.seen = False
                 return maxsize
 
+        # jump=False: non-taken path of jumps, or any non-jump
+        size, maxsize = update_size(instr.stack_effect(jump=False),
+                                    size, maxsize)
     if block.next_block:
         maxsize = _compute_stack_size(block.next_block, size, maxsize)
 
@@ -104,6 +136,14 @@ class ControlFlowGraph(_bytecode.BaseBytecode):
 
         self.add_block()
 
+    def legalize(self):
+        """Legalize all blocks.
+
+        """
+        current_lineno = self.first_lineno
+        for block in self._blocks:
+            current_lineno = block.legalize(current_lineno)
+
     def get_block_index(self, block):
         try:
             return self._block_index[id(block)]
@@ -319,3 +359,10 @@ class ControlFlowGraph(_bytecode.BaseBytecode):
         bytecode[:] = instructions
 
         return bytecode
+
+    def to_code(self, stacksize=None):
+        """Convert to code."""
+        if stacksize is None:
+            stacksize = self.compute_stacksize()
+        bc = self.to_bytecode()
+        return bc.to_code(stacksize=stacksize)
