@@ -132,7 +132,7 @@ class ConcreteInstr(Instr):
         return cls(name, arg, lineno=lineno)
 
 
-class ConcreteBytecode(_bytecode.BaseBytecode, list):
+class ConcreteBytecode(_bytecode._BaseBytecodeList):
 
     def __init__(self, instructions=(), *, consts=(), names=(), varnames=()):
         super().__init__()
@@ -156,6 +156,13 @@ class ConcreteBytecode(_bytecode.BaseBytecode, list):
                              "but %s was found"
                              % type(instr).__name__)
 
+    def _copy_attr_from(self, bytecode):
+        super()._copy_attr_from(bytecode)
+        if isinstance(bytecode, ConcreteBytecode):
+            self.consts = bytecode.consts
+            self.names = bytecode.names
+            self.varnames = bytecode.varnames
+
     def __repr__(self):
         return '<ConcreteBytecode instr#=%s>' % len(self)
 
@@ -240,6 +247,8 @@ class ConcreteBytecode(_bytecode.BaseBytecode, list):
         bytecode.filename = code.co_filename
         bytecode.flags = code.co_flags
         bytecode.argcount = code.co_argcount
+        if sys.version_info >= (3, 8):
+            bytecode.posonlyargcount = code.co_posonlyargcount
         bytecode.kwonlyargcount = code.co_kwonlyargcount
         bytecode.first_lineno = code.co_firstlineno
         bytecode.names = list(code.co_names)
@@ -316,26 +325,46 @@ class ConcreteBytecode(_bytecode.BaseBytecode, list):
         cfg = _bytecode.ControlFlowGraph.from_bytecode(bytecode)
         return cfg.compute_stacksize()
 
-    def to_code(self):
+    def to_code(self, stacksize=None):
         code_str, linenos = self._assemble_code()
         lnotab = self._assemble_lnotab(self.first_lineno, linenos)
         nlocals = len(self.varnames)
-        stacksize = self.compute_stacksize()
-        return types.CodeType(self.argcount,
-                              self.kwonlyargcount,
-                              nlocals,
-                              stacksize,
-                              int(self.flags),
-                              code_str,
-                              tuple(self.consts),
-                              tuple(self.names),
-                              tuple(self.varnames),
-                              self.filename,
-                              self.name,
-                              self.first_lineno,
-                              lnotab,
-                              tuple(self.freevars),
-                              tuple(self.cellvars))
+        if stacksize is None:
+            stacksize = self.compute_stacksize()
+
+        if sys.version_info < (3, 8):
+            return types.CodeType(self.argcount,
+                                  self.kwonlyargcount,
+                                  nlocals,
+                                  stacksize,
+                                  int(self.flags),
+                                  code_str,
+                                  tuple(self.consts),
+                                  tuple(self.names),
+                                  tuple(self.varnames),
+                                  self.filename,
+                                  self.name,
+                                  self.first_lineno,
+                                  lnotab,
+                                  tuple(self.freevars),
+                                  tuple(self.cellvars))
+        else:
+            return types.CodeType(self.argcount,
+                                  self.posonlyargcount,
+                                  self.kwonlyargcount,
+                                  nlocals,
+                                  stacksize,
+                                  int(self.flags),
+                                  code_str,
+                                  tuple(self.consts),
+                                  tuple(self.names),
+                                  tuple(self.varnames),
+                                  self.filename,
+                                  self.name,
+                                  self.first_lineno,
+                                  lnotab,
+                                  tuple(self.freevars),
+                                  tuple(self.cellvars))
 
     def to_bytecode(self):
         # find jump targets
@@ -404,6 +433,8 @@ class ConcreteBytecode(_bytecode.BaseBytecode, list):
         bytecode._copy_attr_from(self)
 
         nargs = bytecode.argcount + bytecode.kwonlyargcount
+        if sys.version_info > (3, 8):
+            nargs += bytecode.posonlyargcount
         if bytecode.flags & inspect.CO_VARARGS:
             nargs += 1
         if bytecode.flags & inspect.CO_VARKEYWORDS:
@@ -431,16 +462,18 @@ class _ConvertBytecodeToConcrete:
         self.labels = {}
 
         # used to build ConcreteBytecode() object
-        self.consts = {}
+        self.consts_indices = {}
+        self.consts_list = []
         self.names = []
         self.varnames = []
 
     def add_const(self, value):
         key = const_key(value)
-        if key in self.consts:
-            return self.consts[key]
-        index = len(self.consts)
-        self.consts[key] = index
+        if key in self.consts_indices:
+            return self.consts_indices[key]
+        index = len(self.consts_indices)
+        self.consts_indices[key] = index
+        self.consts_list.append(value)
         return index
 
     @staticmethod
@@ -548,14 +581,9 @@ class _ConvertBytecodeToConcrete:
                                " %d passes"
                                % (pas + 1))
 
-        consts = [None] * len(self.consts)
-        for item, index in self.consts.items():
-            # const_key(value)[1] is value: see const_key() function
-            consts[index] = item[1]
-
         concrete = ConcreteBytecode(
             self.instructions,
-            consts=consts,
+            consts=self.consts_list.copy(),
             names=self.names,
             varnames=self.varnames
         )
