@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 import io
+import sys
 import unittest
 import contextlib
 from bytecode import (Label, Compare, SetLineno, Instr,
@@ -33,6 +34,8 @@ class BlockTests(unittest.TestCase):
         block.append(Label())
         with self.assertRaises(ValueError):
             list(block)
+        with self.assertRaises(ValueError):
+            block.legalize(1)
 
         # Only one jump allowed and only at the end
         block = BasicBlock()
@@ -41,6 +44,8 @@ class BlockTests(unittest.TestCase):
                       Instr('NOP')])
         with self.assertRaises(ValueError):
             list(block)
+        with self.assertRaises(ValueError):
+            block.legalize(1)
 
         # jump target must be a BasicBlock
         block = BasicBlock()
@@ -48,6 +53,22 @@ class BlockTests(unittest.TestCase):
         block.extend([Instr('JUMP_ABSOLUTE', label)])
         with self.assertRaises(ValueError):
             list(block)
+        with self.assertRaises(ValueError):
+            block.legalize(1)
+
+    def test_slice(self):
+        block = BasicBlock([Instr("NOP")])
+        next_block = BasicBlock()
+        block.next_block = next_block
+        self.assertEqual(block, block[:])
+        self.assertIs(next_block, block[:].next_block)
+
+    def test_copy(self):
+        block = BasicBlock([Instr("NOP")])
+        next_block = BasicBlock()
+        block.next_block = next_block
+        self.assertEqual(block, block.copy())
+        self.assertIs(next_block, block.copy().next_block)
 
 
 class BytecodeBlocksTests(TestCase):
@@ -73,6 +94,8 @@ class BytecodeBlocksTests(TestCase):
         self.assertEqual(code.argcount, 2)
         self.assertEqual(code.filename, "hello.py")
         self.assertEqual(code.first_lineno, 3)
+        if sys.version_info > (3, 8):
+            self.assertEqual(code.posonlyargcount, 0)
         self.assertEqual(code.kwonlyargcount, 1)
         self.assertEqual(code.name, "func")
         self.assertEqual(code.cellvars, [])
@@ -132,6 +155,32 @@ class BytecodeBlocksTests(TestCase):
                                 Instr("LOAD_CONST", 9),
                                 Instr("STORE_NAME", 'z')])
 
+    def test_legalize(self):
+        code = Bytecode()
+        code.first_lineno = 3
+        code.extend([Instr("LOAD_CONST", 7),
+                     Instr("STORE_NAME", 'x'),
+                     Instr("LOAD_CONST", 8, lineno=4),
+                     Instr("STORE_NAME", 'y'),
+                     SetLineno(5),
+                     Instr("LOAD_CONST", 9, lineno=6),
+                     Instr("STORE_NAME", 'z')])
+
+        blocks = ControlFlowGraph.from_bytecode(code)
+        blocks.legalize()
+        self.assertBlocksEqual(blocks,
+                               [Instr("LOAD_CONST", 7, lineno=3),
+                                Instr("STORE_NAME", 'x', lineno=3),
+                                Instr("LOAD_CONST", 8, lineno=4),
+                                Instr("STORE_NAME", 'y', lineno=4),
+                                Instr("LOAD_CONST", 9, lineno=5),
+                                Instr("STORE_NAME", 'z', lineno=5)])
+
+    def test_repr(self):
+        r = repr(ControlFlowGraph())
+        self.assertIn("ControlFlowGraph", r)
+        self.assertIn("1", r)
+
     def test_to_bytecode(self):
         # if test:
         #     x = 2
@@ -221,60 +270,108 @@ class BytecodeBlocksTests(TestCase):
         #         break
         #     continue
 
-        label_loop_start = Label()
-        label_loop_exit = Label()
-        label_loop_end = Label()
+        if sys.version_info < (3, 8):
+            label_loop_start = Label()
+            label_loop_exit = Label()
+            label_loop_end = Label()
 
-        code = Bytecode()
-        code.extend((Instr('SETUP_LOOP', label_loop_end, lineno=1),
-                     Instr('LOAD_CONST', (1, 2, 3), lineno=1),
-                     Instr('GET_ITER', lineno=1),
+            code = Bytecode()
+            code.extend((Instr('SETUP_LOOP', label_loop_end, lineno=1),
+                         Instr('LOAD_CONST', (1, 2, 3), lineno=1),
+                         Instr('GET_ITER', lineno=1),
 
-                     label_loop_start,
-                     Instr('FOR_ITER', label_loop_exit, lineno=1),
-                     Instr('STORE_NAME', 'x', lineno=1),
-                     Instr('LOAD_NAME', 'x', lineno=2),
-                     Instr('LOAD_CONST', 2, lineno=2),
-                     Instr('COMPARE_OP', Compare.EQ, lineno=2),
-                     Instr('POP_JUMP_IF_FALSE', label_loop_start, lineno=2),
-                     Instr('BREAK_LOOP', lineno=3),
-                     Instr('JUMP_ABSOLUTE', label_loop_start, lineno=4),
+                         label_loop_start,
+                         Instr('FOR_ITER', label_loop_exit, lineno=1),
+                         Instr('STORE_NAME', 'x', lineno=1),
+                         Instr('LOAD_NAME', 'x', lineno=2),
+                         Instr('LOAD_CONST', 2, lineno=2),
+                         Instr('COMPARE_OP', Compare.EQ, lineno=2),
+                         Instr('POP_JUMP_IF_FALSE', label_loop_start, lineno=2),
+                         Instr('BREAK_LOOP', lineno=3),
+                         Instr('JUMP_ABSOLUTE', label_loop_start, lineno=4),
 
-                     Instr('JUMP_ABSOLUTE', label_loop_start, lineno=4),
+                         Instr('JUMP_ABSOLUTE', label_loop_start, lineno=4),
 
-                     label_loop_exit,
-                     Instr('POP_BLOCK', lineno=4),
+                         label_loop_exit,
+                         Instr('POP_BLOCK', lineno=4),
 
-                     label_loop_end,
-                     Instr('LOAD_CONST', None, lineno=4),
-                     Instr('RETURN_VALUE', lineno=4),
-                     ))
-        blocks = ControlFlowGraph.from_bytecode(code)
+                         label_loop_end,
+                         Instr('LOAD_CONST', None, lineno=4),
+                         Instr('RETURN_VALUE', lineno=4),
+                         ))
+            blocks = ControlFlowGraph.from_bytecode(code)
 
-        expected = [[Instr('SETUP_LOOP', blocks[8], lineno=1)],
+            expected = [[Instr('SETUP_LOOP', blocks[8], lineno=1)],
 
-                    [Instr('LOAD_CONST', (1, 2, 3), lineno=1),
-                     Instr('GET_ITER', lineno=1)],
+                        [Instr('LOAD_CONST', (1, 2, 3), lineno=1),
+                         Instr('GET_ITER', lineno=1)],
 
-                    [Instr('FOR_ITER', blocks[7], lineno=1)],
+                        [Instr('FOR_ITER', blocks[7], lineno=1)],
 
-                    [Instr('STORE_NAME', 'x', lineno=1),
-                     Instr('LOAD_NAME', 'x', lineno=2),
-                     Instr('LOAD_CONST', 2, lineno=2),
-                     Instr('COMPARE_OP', Compare.EQ, lineno=2),
-                     Instr('POP_JUMP_IF_FALSE', blocks[2], lineno=2)],
+                        [Instr('STORE_NAME', 'x', lineno=1),
+                         Instr('LOAD_NAME', 'x', lineno=2),
+                         Instr('LOAD_CONST', 2, lineno=2),
+                         Instr('COMPARE_OP', Compare.EQ, lineno=2),
+                         Instr('POP_JUMP_IF_FALSE', blocks[2], lineno=2)],
 
-                    [Instr('BREAK_LOOP', lineno=3)],
+                        [Instr('BREAK_LOOP', lineno=3)],
 
-                    [Instr('JUMP_ABSOLUTE', blocks[2], lineno=4)],
+                        [Instr('JUMP_ABSOLUTE', blocks[2], lineno=4)],
 
-                    [Instr('JUMP_ABSOLUTE', blocks[2], lineno=4)],
+                        [Instr('JUMP_ABSOLUTE', blocks[2], lineno=4)],
 
-                    [Instr('POP_BLOCK', lineno=4)],
+                        [Instr('POP_BLOCK', lineno=4)],
 
-                    [Instr('LOAD_CONST', None, lineno=4),
-                     Instr('RETURN_VALUE', lineno=4)]]
-        self.assertBlocksEqual(blocks, *expected)
+                        [Instr('LOAD_CONST', None, lineno=4),
+                         Instr('RETURN_VALUE', lineno=4)]]
+            self.assertBlocksEqual(blocks, *expected)
+        else:
+            label_loop_start = Label()
+            label_loop_exit = Label()
+
+            code = Bytecode()
+            code.extend((Instr('LOAD_CONST', (1, 2, 3), lineno=1),
+                         Instr('GET_ITER', lineno=1),
+
+                         label_loop_start,
+                         Instr('FOR_ITER', label_loop_exit, lineno=1),
+                         Instr('STORE_NAME', 'x', lineno=1),
+                         Instr('LOAD_NAME', 'x', lineno=2),
+                         Instr('LOAD_CONST', 2, lineno=2),
+                         Instr('COMPARE_OP', Compare.EQ, lineno=2),
+                         Instr('POP_JUMP_IF_FALSE', label_loop_start, lineno=2),
+                         Instr('JUMP_ABSOLUTE', label_loop_exit, lineno=3),
+
+                         Instr('JUMP_ABSOLUTE', label_loop_start, lineno=4),
+
+                         Instr('JUMP_ABSOLUTE', label_loop_start, lineno=4),
+
+                         label_loop_exit,
+                         Instr('LOAD_CONST', None, lineno=4),
+                         Instr('RETURN_VALUE', lineno=4),
+                         ))
+            blocks = ControlFlowGraph.from_bytecode(code)
+
+            expected = [[Instr('LOAD_CONST', (1, 2, 3), lineno=1),
+                         Instr('GET_ITER', lineno=1)],
+
+                        [Instr('FOR_ITER', blocks[6], lineno=1)],
+
+                        [Instr('STORE_NAME', 'x', lineno=1),
+                         Instr('LOAD_NAME', 'x', lineno=2),
+                         Instr('LOAD_CONST', 2, lineno=2),
+                         Instr('COMPARE_OP', Compare.EQ, lineno=2),
+                         Instr('POP_JUMP_IF_FALSE', blocks[1], lineno=2)],
+
+                        [Instr('JUMP_ABSOLUTE', blocks[6], lineno=3)],
+
+                        [Instr('JUMP_ABSOLUTE', blocks[1], lineno=4)],
+
+                        [Instr('JUMP_ABSOLUTE', blocks[1], lineno=4)],
+
+                        [Instr('LOAD_CONST', None, lineno=4),
+                         Instr('RETURN_VALUE', lineno=4)]]
+            self.assertBlocksEqual(blocks, *expected)
 
 
 class BytecodeBlocksFunctionalTests(TestCase):
@@ -287,6 +384,19 @@ class BytecodeBlocksFunctionalTests(TestCase):
         code2 = disassemble(source)
         self.assertEqual(code1, code2)
 
+        # Type mismatch
+        self.assertFalse(code1 == 1)
+
+        # argnames mismatch
+        cfg = ControlFlowGraph()
+        cfg.argnames = 10
+        self.assertFalse(code1 == cfg)
+
+        # instr mismatch
+        cfg = ControlFlowGraph()
+        cfg.argnames = code1.argnames
+        self.assertFalse(code1 == cfg)
+
     def check_getitem(self, code):
         # check internal Code block indexes (index by index, index by label)
         for block_index, block in enumerate(code):
@@ -294,6 +404,12 @@ class BytecodeBlocksFunctionalTests(TestCase):
             self.assertIs(code[block], block)
             self.assertEqual(code.get_block_index(block), block_index)
 
+    def test_delitem(self):
+        cfg = ControlFlowGraph()
+        b = cfg.add_block()
+        del cfg[b]
+        self.assertEqual(len(cfg.get_instructions()), 0)
+
     def sample_code(self):
         code = disassemble('x = 1', remove_last_return_none=True)
         self.assertBlocksEqual(code,
@@ -321,6 +437,13 @@ class BytecodeBlocksFunctionalTests(TestCase):
                                [Instr('NOP', lineno=1)])
         self.check_getitem(code)
 
+        with self.assertRaises(TypeError):
+            code.split_block(1, 1)
+
+        with self.assertRaises(ValueError) as e:
+            code.split_block(code[0], -2)
+        self.assertIn("positive", e.exception.args[0])
+
     def test_split_block_end(self):
         code = self.sample_code()
 
@@ -364,6 +487,8 @@ class BytecodeBlocksFunctionalTests(TestCase):
         bytecode = ControlFlowGraph()
         bytecode.first_lineno = 3
         bytecode.argcount = 3
+        if sys.version_info > (3, 8):
+            bytecode.posonlyargcount = 0
         bytecode.kwonlyargcount = 2
         bytecode.name = 'func'
         bytecode.filename = 'hello.py'
@@ -401,9 +526,11 @@ class BytecodeBlocksFunctionalTests(TestCase):
                         b'|\x05\x00'
                         b'S')
 
-        code = bytecode.to_bytecode().to_code()
+        code = bytecode.to_code()
         self.assertEqual(code.co_consts, (None, 3))
         self.assertEqual(code.co_argcount, 3)
+        if sys.version_info > (3, 8):
+            self.assertEqual(code.co_posonlyargcount, 0)
         self.assertEqual(code.co_kwonlyargcount, 2)
         self.assertEqual(code.co_nlocals, 6)
         self.assertEqual(code.co_stacksize, 1)
@@ -417,6 +544,11 @@ class BytecodeBlocksFunctionalTests(TestCase):
         self.assertEqual(code.co_name, 'func')
         self.assertEqual(code.co_firstlineno, 3)
 
+        # verify stacksize argument is honored
+        explicit_stacksize = code.co_stacksize + 42
+        code = bytecode.to_code(stacksize=explicit_stacksize)
+        self.assertEqual(code.co_stacksize, explicit_stacksize)
+
     def test_get_block_index(self):
         blocks = ControlFlowGraph()
         block0 = blocks[0]
@@ -439,7 +571,9 @@ class CFGStacksizeComputationTests(TestCase):
         self.assertEqual(code.co_stacksize, cfg.compute_stacksize())
 
     def test_empty_code(self):
-        self.assertEqual(ControlFlowGraph().compute_stacksize(), 0)
+        cfg = ControlFlowGraph()
+        del cfg[0]
+        self.assertEqual(cfg.compute_stacksize(), 0)
 
     def test_handling_of_set_lineno(self):
         code = Bytecode()
@@ -516,8 +650,6 @@ class CFGStacksizeComputationTests(TestCase):
             with open(arg1) as f:
                 return f.read()
 
-        import dis
-        dis.dis(test.__code__)
         self.check_stack_size(test)
 
     def test_stack_size_computation_try_except(self):
