@@ -5,7 +5,7 @@ import types
 import unittest
 import textwrap
 from bytecode import (UNSET, Label, Instr, SetLineno, Bytecode,
-                      CellVar, FreeVar,
+                      CellVar, FreeVar, CompilerFlags,
                       ConcreteInstr, ConcreteBytecode)
 from bytecode.tests import get_code, TestCase, WORDCODE
 
@@ -162,6 +162,39 @@ class ConcreteInstrTests(TestCase):
 
 class ConcreteBytecodeTests(TestCase):
 
+    def test_repr(self):
+        r = repr(ConcreteBytecode())
+        self.assertIn("ConcreteBytecode", r)
+        self.assertIn("0", r)
+
+    def test_eq(self):
+        code = ConcreteBytecode()
+        self.assertFalse(code == 1)
+
+        for name, val in (("names", ["a"]), ("varnames", ["a"]),
+                          ("consts", [1]),
+                          ("argcount", 1), ("kwonlyargcount", 2),
+                          ("flags", CompilerFlags(CompilerFlags.GENERATOR)),
+                          ("first_lineno", 10), ("filename", "xxxx.py"),
+                          ("name", "__x"), ("docstring", "x-x-x"),
+                          ("cellvars", [CellVar("x")]),
+                          ("freevars", [FreeVar("x")])):
+            c = ConcreteBytecode()
+            setattr(c, name, val)
+            # For obscure reasons using assertNotEqual here fail
+            self.assertFalse(code == c)
+
+        if sys.version_info > (3, 8):
+            c = ConcreteBytecode()
+            c.posonlyargcount = 10
+            self.assertFalse(code == c)
+
+        c = ConcreteBytecode()
+        c.consts = [1]
+        code.consts = [1]
+        c.append(ConcreteInstr("LOAD_CONST", 0))
+        self.assertFalse(code == c)
+
     def test_attr(self):
         code_obj = get_code("x = 5")
         code = ConcreteBytecode.from_code(code_obj)
@@ -181,6 +214,8 @@ class ConcreteBytecodeTests(TestCase):
         code.append(Label())
         with self.assertRaises(ValueError):
             list(code)
+        with self.assertRaises(ValueError):
+            code.legalize()
         with self.assertRaises(ValueError):
             ConcreteBytecode([Label()])
 
@@ -359,6 +394,77 @@ class ConcreteBytecodeTests(TestCase):
             code.co_code,
             b'\x94\x01\x89\x01' if WORDCODE else b'\x94\x01\x00\x89\x01\x00')
 
+    def test_explicit_stacksize(self):
+        # Passing stacksize=... to ConcreteBytecode.to_code should result in a
+        # code object with the specified stacksize.  We pass some silly values
+        # and assert that they are honored.
+        code_obj = get_code("print('%s' % (a,b,c))")
+        original_stacksize = code_obj.co_stacksize
+        concrete = ConcreteBytecode.from_code(code_obj)
+
+        # First with something bigger than necessary.
+        explicit_stacksize = original_stacksize + 42
+        new_code_obj = concrete.to_code(stacksize=explicit_stacksize)
+        self.assertEqual(new_code_obj.co_stacksize, explicit_stacksize)
+
+        # Then with something bogus.  We probably don't want to advertise this
+        # in the documentation.  If this fails then decide if it's for good
+        # reason, and remove if so.
+        explicit_stacksize = 0
+        new_code_obj = concrete.to_code(stacksize=explicit_stacksize)
+        self.assertEqual(new_code_obj.co_stacksize, explicit_stacksize)
+
+    def test_legalize(self):
+        concrete = ConcreteBytecode()
+        concrete.first_lineno = 3
+        concrete.consts = [7, 8, 9]
+        concrete.names = ['x', 'y', 'z']
+        concrete.extend([ConcreteInstr("LOAD_CONST", 0),
+                         ConcreteInstr("STORE_NAME", 0),
+                         ConcreteInstr("LOAD_CONST", 1, lineno=4),
+                         ConcreteInstr("STORE_NAME", 1),
+                         SetLineno(5),
+                         ConcreteInstr("LOAD_CONST", 2, lineno=6),
+                         ConcreteInstr("STORE_NAME", 2)])
+
+        concrete.legalize()
+        self.assertListEqual(list(concrete), [ConcreteInstr("LOAD_CONST", 0, lineno=3),
+                                              ConcreteInstr("STORE_NAME", 0, lineno=3),
+                                              ConcreteInstr("LOAD_CONST", 1, lineno=4),
+                                              ConcreteInstr("STORE_NAME", 1, lineno=4),
+                                              ConcreteInstr("LOAD_CONST", 2, lineno=5),
+                                              ConcreteInstr("STORE_NAME", 2, lineno=5)])
+
+    def test_slice(self):
+        concrete = ConcreteBytecode()
+        concrete.first_lineno = 3
+        concrete.consts = [7, 8, 9]
+        concrete.names = ['x', 'y', 'z']
+        concrete.extend([ConcreteInstr("LOAD_CONST", 0),
+                         ConcreteInstr("STORE_NAME", 0),
+                         SetLineno(4),
+                         ConcreteInstr("LOAD_CONST", 1),
+                         ConcreteInstr("STORE_NAME", 1),
+                         SetLineno(5),
+                         ConcreteInstr("LOAD_CONST", 2),
+                         ConcreteInstr("STORE_NAME", 2)])
+        self.assertEqual(concrete, concrete[:])
+
+    def test_copy(self):
+        concrete = ConcreteBytecode()
+        concrete.first_lineno = 3
+        concrete.consts = [7, 8, 9]
+        concrete.names = ['x', 'y', 'z']
+        concrete.extend([ConcreteInstr("LOAD_CONST", 0),
+                         ConcreteInstr("STORE_NAME", 0),
+                         SetLineno(4),
+                         ConcreteInstr("LOAD_CONST", 1),
+                         ConcreteInstr("STORE_NAME", 1),
+                         SetLineno(5),
+                         ConcreteInstr("LOAD_CONST", 2),
+                         ConcreteInstr("STORE_NAME", 2)])
+        self.assertEqual(concrete, concrete.copy())
+
 
 class ConcreteFromCodeTests(TestCase):
 
@@ -367,21 +473,25 @@ class ConcreteFromCodeTests(TestCase):
         co_code = (b'\x90\x12\x904\x90\xabd\xcd' if WORDCODE else
                    b'\x904\x12d\xcd\xab')
         code = get_code('x=1')
-        code = types.CodeType(code.co_argcount,
-                              code.co_kwonlyargcount,
-                              code.co_nlocals,
-                              code.co_stacksize,
-                              code.co_flags,
-                              co_code,
-                              code.co_consts,
-                              code.co_names,
-                              code.co_varnames,
-                              code.co_filename,
-                              code.co_name,
-                              code.co_firstlineno,
-                              code.co_lnotab,
-                              code.co_freevars,
-                              code.co_cellvars)
+        args = ((code.co_argcount,)
+                if sys.version_info < (3, 8) else
+                (code.co_argcount, code.co_posonlyargcount))
+        args += (code.co_kwonlyargcount,
+                 code.co_nlocals,
+                 code.co_stacksize,
+                 code.co_flags,
+                 co_code,
+                 code.co_consts,
+                 code.co_names,
+                 code.co_varnames,
+                 code.co_filename,
+                 code.co_name,
+                 code.co_firstlineno,
+                 code.co_lnotab,
+                 code.co_freevars,
+                 code.co_cellvars)
+
+        code = types.CodeType(*args)
 
         # without EXTENDED_ARG opcode
         bytecode = ConcreteBytecode.from_code(code)
@@ -793,6 +903,35 @@ class BytecodeToConcreteTests(TestCase):
 
         code.to_code(compute_jumps_passes=(len(labels) + 1))
 
+    def test_general_constants(self):
+        """Test if general object could be linked as constants.
+
+        """
+        class CustomObject:
+            pass
+
+        class UnHashableCustomObject:
+            __hash__ = None
+
+        obj1 = [1, 2, 3]
+        obj2 = {1, 2, 3}
+        obj3 = CustomObject()
+        obj4 = UnHashableCustomObject()
+        code = Bytecode([Instr('LOAD_CONST', obj1, lineno=1),
+                         Instr('LOAD_CONST', obj2, lineno=1),
+                         Instr('LOAD_CONST', obj3, lineno=1),
+                         Instr('LOAD_CONST', obj4, lineno=1),
+                         Instr('BUILD_TUPLE', 4, lineno=1),
+                         Instr('RETURN_VALUE', lineno=1)])
+        self.assertEqual(code.to_code().co_consts,
+                         (obj1, obj2, obj3, obj4))
+
+        def f():
+            return  # pragma: no cover
+
+        f.__code__ = code.to_code()
+        self.assertEqual(f(), (obj1, obj2, obj3, obj4))
+
 
 if __name__ == "__main__":
-    unittest.main()
+    unittest.main()  # pragma: no cover
